/**
 * This file is part of osso-email-search-plugin
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#include "ogs-email-plugin-modest.h"
#include "libmodest-dbus-client.h"

extern osso_context_t *email_plugin_ctx;

void
email_plugin_query_modest (OgsPlugin         *plugin,
			   const gchar       *query,
			   OgsSearchContext  *context,
			   OgsFileSystem     *fs,
			   OgsSearchCategory  category,
			   OgsSearchOptions  *options)
{
	ModestDBusSearchFlags  flags;
	OgsSearchFields    fields;
	OgsSearchHit      *hit;
	const gchar       *folder;
	time_t             start_date;
	time_t             end_date;
	gboolean           ret;
	GList             *hits;
	GList             *iter;
	gint               scope;
	guint32            size;

	osso_log (LOG_INFO, "Email Plugin Query Starts here %s \n", query);

	if (!ogs_plugin_supports_category (plugin, category)) {
		osso_log (LOG_INFO, "Invalid Category\n");
		return;
	}

	start_date = 0;
	end_date   = 0;
	size       = 0;
	flags      = 0;
	hits       = NULL;

	fields = ogs_search_options_get_valid_fields (options);

	scope = ogs_search_options_get_scope (options);

	switch (scope) {

	case OGS_SEARCH_SCOPE_ALL:	        /* Entire body and attachment */
		flags |= MODEST_DBUS_SEARCH_SUBJECT |
			     MODEST_DBUS_SEARCH_SENDER |
				 MODEST_DBUS_SEARCH_RECIPIENT |
				 MODEST_DBUS_SEARCH_BODY;
		break;

	case OGS_SEARCH_SCOPE_ADDRESSES:	/* From, To, CC fileds */
		flags |= MODEST_DBUS_SEARCH_SUBJECT |
			     MODEST_DBUS_SEARCH_SENDER |
				 MODEST_DBUS_SEARCH_RECIPIENT;
		break;

	case OGS_SEARCH_SCOPE_TITLES:	    /* Titles, subjects only */
		flags |= MODEST_DBUS_SEARCH_SUBJECT;
		break;

	case OGS_SEARCH_SCOPE_CONTENTS:	    /* message bodies */
		flags |= MODEST_DBUS_SEARCH_BODY;
		break;

	default:
		osso_log (LOG_INFO, "Invalid Search option = %d \n", scope);
		return;
	}

	folder = ogs_search_options_get_folder (options);

	if (fields & OGS_SEARCH_OPTIONS_MIN_SIZE) {
		size = ogs_search_options_get_min_size (options);
	}

	if (fields & OGS_SEARCH_OPTIONS_DATE_START) {
		const GDate *date;
		
		date = ogs_search_options_get_date_start (options);

		if (date != NULL) {
			struct tm tm;
			g_date_to_struct_tm (date,
								 &tm);

			start_date = mktime (&tm);
		}
	}

	if (fields & OGS_SEARCH_OPTIONS_DATE_END) {
		const GDate *date;
		date = ogs_search_options_get_date_end (options);
	
		if (date != NULL) {
			struct tm tm;
			g_date_to_struct_tm (date,
								 &tm);

			end_date = mktime (&tm);
		}
	}

	ret = libmodest_dbus_client_search (email_plugin_ctx,
					    query,
					    folder,
					    start_date,
					    end_date,
					    size,
					    flags,
					    &hits);

	if (ret == FALSE) {
		osso_log (LOG_INFO, "Error during search");
		return;
	}


	for (iter = hits; iter; iter = iter->next) {
		ModestSearchHit *mh = (ModestSearchHit *) iter->data;
				
		hit = g_object_new (OGS_TYPE_EMAIL_HIT, NULL);
		if (!hit) {
			g_warning ("%s: failed to create hit", __FUNCTION__);
			continue;
		}

		ogs_search_hit_set_category  (hit, OGS_SEARCH_CATEGORY_EMAIL);
		ogs_search_hit_set_mime_type (hit, "application/x-modest");

		ogs_email_hit_set_contact        (OGS_EMAIL_HIT(hit),  mh->sender);
		ogs_email_hit_set_has_attachment (OGS_EMAIL_HIT(hit),  mh->has_attachment);
		ogs_email_hit_set_is_unread      (OGS_EMAIL_HIT(hit),  mh->is_unread);
		ogs_email_hit_set_msgid          (OGS_EMAIL_HIT(hit),  mh->msgid);
		
		g_object_set (hit,
			      "name",           mh->subject, 
			      "folder", 	mh->folder, 
			      "timestamp",      mh->timestamp,
			      "size",           mh->msize,  
			      NULL);
				
		ogs_plugin_new_hit (plugin, hit, context);
		g_object_unref (hit);
	}

	modest_search_hit_list_free (hits);
}


gboolean
email_plugin_delete_hit_modest (OgsPlugin *plugin, OgsSearchHit *hit, GError **error)
{
	OgsEmailHit *email_hit;
	gboolean      ret;
	const gchar *msgid = NULL;

	email_hit = OGS_EMAIL_HIT (hit);
	msgid = ogs_email_hit_get_msgid (email_hit);

	if (msgid == NULL) {
		g_set_error (error, 0, 0, "Could not delete mail");
		return FALSE;
	}

	ret = libmodest_dbus_client_delete_message (email_plugin_ctx,
												msgid);

	if (ret == FALSE) {
		g_set_error (error, 0, 0, "Could not delete mail");
		return FALSE;
	}
	
	return TRUE;
}


gboolean
email_plugin_open_hit_modest (OgsPlugin *plugin, OgsSearchHit *hit, GError **error)
{
	OgsEmailHit *email_hit;
	gboolean      ret;
	const gchar *msgid = NULL;

	email_hit = OGS_EMAIL_HIT (hit);
	msgid = ogs_email_hit_get_msgid (email_hit);

	if (msgid == NULL) {
		g_set_error (error, 0, 0, "Could not open mail");
		return FALSE;
	}

	ret = libmodest_dbus_client_open_message (email_plugin_ctx,
											  msgid);

	if (ret == FALSE) {
		g_set_error (error, 0, 0, "Could not open mail");
		return FALSE;
	}

	return TRUE;

}
