/**
 * This file is part of osso-email-search-plugin
 *
 * Copyright (c) 2006 Nokia Corporation.
 * 
 * Contact: Dirk-Jan Binnema <dirk-jan.binnema@nokia.com>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2.1 as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */


#include <config.h>

#include <string.h>
#include <time.h>
#include <unistd.h>

#include <glib-object.h>

#include <libogs/ogs.h>
#include <libogs/ogs-text-searcher.h>
#include <libogs/ogs-file-system.h>
#include <libogs/ogs-email-hit.h>

#include <gconf/gconf-client.h>
#include "ogs-email.h"

#define MAX_TRANSLATED_FOLDER_NAME_LEN 255

#define OGS_TYPE_EMAIL_PLUGIN   	(ogs_email_plugin_type)
#define OGS_EMAIL_PLUGIN(obj)   	(G_TYPE_CHECK_INSTANCE_CAST ((obj), \
                                 	OGS_TYPE_EMAIL_PLUGIN, \
                                 	OgsEmailPlugin))
#define EMAIL_PLUGIN_CLASS(k)   	(G_TYPE_CHECK_CLASS_CAST((k), \
                                 	OGS_TYPE_EMAIL_PLUGIN, \
                                 	OgsEmailPluginClass))
#define OGS_IS_EMAIL_PLUGIN(obj)  	(G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
	                           	OGS_TYPE_EMAIL_PLUGIN))
#define OGS_IS_EMAIL_PLUGIN_CLASS(k)  	(G_TYPE_CHECK_CLASS_TYPE((k), \
		                     	OGS_TYPE_EMAIL_PLUGIN))
#define OGS_EMAIL_PLUGIN_GET_CLASS(o) 	(G_TYPE_INSTANCE_GET_CLASS ((o), \
                                      	OGS_TYPE_EMAIL_PLUGIN, \
                                    	OgsEmailPluginClass))
#define DGETTEXT_GS(String) dgettext("osso-global-search",("srch_ia_no_subject"))

typedef struct _OgsEmailPlugin OgsEmailPlugin;
typedef struct _OgsEmailPluginClass OgsEmailPluginClass;

struct _OgsEmailPlugin {
	OgsPlugin parent;
};

struct _OgsEmailPluginClass {
	OgsPluginClass parent_class;
};

static GType ogs_email_plugin_type = 0;
static OgsPluginClass *parent_class = NULL;

typedef struct {
	OgsPlugin *plugin;
	OgsSearchContext *context;
	OgsSearchOptions *options;
	OgsTextSearcher *searcher;
	OgsSearchCategory category;
	gchar *query;
	gchar *sel_folder;
} EmailData;

typedef struct {
	gchar *fname;
	gchar *lname;
} EmailStandardFolder;

osso_context_t *email_plugin_ctx = NULL;
static EmailStandardFolder standard_folders[STANFOLDERS_SIZE];

static GType email_plugin_get_type(GTypeModule * module);
static void email_plugin_class_init(OgsEmailPluginClass * class);
static void email_plugin_init(OgsEmailPlugin * plugin);

static void email_plugin_finalize(GObject * object);
static void email_plugin_query(OgsPlugin * plugin,
			       const gchar * query,
			       OgsSearchContext * context,
			       OgsFileSystem * fs,
			       OgsSearchCategory category, OgsSearchOptions * options);

static void email_plugin_add_hit(EmailData * pl_data,
				 const gchar * contact,
				 const gchar * subject,
				 const gchar * folder,
				 const gchar * msgid,
				 glong timestamp,
				 gboolean has_attachment, gboolean is_unread, guint msize);

static gboolean email_plugin_delete_hit(OgsPlugin * plugin, OgsSearchHit * hit, GError ** error);
static gboolean email_plugin_open_hit(OgsPlugin * plugin, OgsSearchHit * hit, GError ** error);

static void email_plugin_start_search(EmailData * search_data);

static void email_plugin_search_sel_folder(EmailData * pl_data, gchar * folder);

static MmcHlrCallback *mmc_init(void);
static void mmc_uninit(MmcHlrCallback * mmc_hlrCb);
static gint mmc_hlr_volume_unmounted_cb(GnomeVFSVolumeMonitor * volume_monitor,
					GnomeVFSVolume * volume);
static gint mmc_hlr_volume_mounted_cb(GnomeVFSVolumeMonitor * volume_monitor,
				      GnomeVFSVolume * volume);

#define MODEST_IS_DEFAULT_KEY "/apps/modest/is_default"

/* check if modest is the default mailer */
static gboolean
modest_is_default (void)
{
	gboolean is_default;
	
	GConfClient *conf = gconf_client_get_default ();
	if (!conf) {
		g_warning ("%s: could not get gconf client", __FUNCTION__);
		return FALSE;
	}
	/* this wil return FALSE in case of any error */
	is_default = gconf_client_get_bool (conf, MODEST_IS_DEFAULT_KEY, NULL);
	
	if (conf)
		g_object_unref (conf);

	return is_default;
}



/**
	Email Plugin Module is required to implemenet this function. 
	This function  will be called by ligogs library at the time 
	of module loading.

	@param module is added with email plugin type module (Email)
	@return nothing
*/

G_MODULE_EXPORT void ogs_module_load(OgsModule * module)
{
	email_plugin_get_type(G_TYPE_MODULE(module));
}

/**
	Email Plugin Module is required to implemenet this function. 
	This function  will be called by ligogs library at the time 
	of module loading.

	@param module is OgsModule for which email plugin type registered. 
	@return nothing
*/

G_MODULE_EXPORT void ogs_module_query(OgsModule * module)
{
	ogs_search_module_register_plugin(OGS_SEARCH_MODULE(module),
					  OGS_SEARCH_CATEGORY_EMAIL, ogs_email_plugin_type);
}

/**
	Email Plugin Module is required to implemenet this function. 
	This function will be called by ligogs library at the time of 
	unloading the email plugin.

	@param module contains OgsType module
	@return nothing
*/

G_MODULE_EXPORT void ogs_module_unload(OgsModule * module)
{
/* Nothing can be done here */
}

/**
	This method will be called to register email plugin info.  
	@param module Email plugin module is populated with email search plugin data
	@return  the email plugin type  
*/

static GType email_plugin_get_type(GTypeModule * module)
{
	if(!module) {
		osso_log(LOG_INFO, "GType module is null \n");
	}
	if (!ogs_email_plugin_type) {
		static const GTypeInfo plugin_info = {
			sizeof(OgsEmailPluginClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) email_plugin_class_init,
			NULL,	/* class_finalize */
			NULL,	/* class_data     */
			sizeof(OgsEmailPlugin),
			0,	/* n_preallocs    */
			(GInstanceInitFunc) email_plugin_init
		};

		ogs_email_plugin_type =
		    g_type_module_register_type(module,
						OGS_TYPE_PLUGIN, "EmailPlugin", &plugin_info, 0);
	}
	return ogs_email_plugin_type;
}

/**
	This method is called to initialse email plugin class and populate
	the query, open, delete methods for Email search plugin.
	@param Email Plugin Class 
	@return nothing 
*/

static void email_plugin_class_init(OgsEmailPluginClass * class)
{
	OgsPluginClass *plugin_class;
	GObjectClass *object_class;

	plugin_class = OGS_PLUGIN_CLASS(class);
	object_class = G_OBJECT_CLASS(class);

	parent_class = g_type_class_peek_parent(class);

	plugin_class->query = email_plugin_query;
	plugin_class->options_type = OGS_TYPE_SEARCH_OPTIONS;
	plugin_class->hit_type = OGS_TYPE_EMAIL_HIT;
	plugin_class->open_hit = email_plugin_open_hit;
	plugin_class->open_hits = NULL;
	plugin_class->delete_hit = email_plugin_delete_hit;
	object_class->finalize = email_plugin_finalize;
}

/**
	This method is called to initialse email plugin object categaory 
	and create osso context for email search plugin 
	@param plugin  Email Search plugin 
*/

static void email_plugin_init(OgsEmailPlugin * plugin)
{
	g_object_set(plugin, "id", "email-plugin", "category", OGS_SEARCH_CATEGORY_EMAIL, NULL);
	/* Create a osso EMail plugin context */
	email_plugin_ctx = osso_initialize(OGS_EMAIL_SEARCH_PLUGIN,
					   OGS_EMAIL_SEARCH_PLUGIN_VERSION, TRUE, NULL);
	if (email_plugin_ctx == NULL) {
		osso_log(LOG_ERR, "Email Search context creation Error \n");
		return;
	}
}

/**
	This method is called to un intialise email search plugin osso context
       	and free the static date allocated.
	@return nothing 
*/

static void email_plugin_finalize(GObject * object)
{
	gint idx = 0;
	if (email_plugin_ctx != NULL) {
		osso_deinitialize(email_plugin_ctx);
	}
	/* Free folder names stored locally */
	for (idx = 0; idx < STANFOLDERS_SIZE; idx++) {
		g_free(standard_folders[idx].fname);
		g_free(standard_folders[idx].lname);
		standard_folders[idx].fname = NULL;
		standard_folders[idx].lname = NULL;
	}

	G_OBJECT_CLASS(parent_class)->finalize(object);
}

/**
	This method is called to get the standard forlder names used in 
	email Application.  It sends a Dbus message and gets the folder 
	data and populates in standard folder struct.

	@return new email plugin object is returned
*/

static void ogs_email_get_email_standardfolders(void)
{
	gchar **folders = NULL;
	gint idx = 0;
	DBusMessage *msg = NULL, *ret = NULL;
	DBusError err;
	DBusConnection *conn = NULL;
	DBusMessageIter iter;
	const gchar *flmsg = NULL;
	dbus_uint32_t reason = 0;
	static gboolean stan_folders_available = FALSE;	/* Send request once */

	if (!email_plugin_ctx ) {
		return;
	}
	if (stan_folders_available == TRUE) {
		return;
	}

	dbus_error_init(&err);
	conn = (DBusConnection *) osso_get_dbus_connection(email_plugin_ctx);
	if(!dbus_bus_start_service_by_name(conn, OSSO_EMAIL_SERVICE, 0, &reason, &err)) {
		return;
	}
	msg = dbus_message_new_method_call(OSSO_EMAIL_SERVICE,
					   OSSO_EMAIL_OBJ_PATH,
					   OSSO_EMAIL_SERVICE, GET_STANDARD_FOLDERS_METHOD);
	if (msg == NULL) {
		dbus_error_free(&err);
		return;
	}
	dbus_message_set_no_reply(msg, FALSE);
	ret = dbus_connection_send_with_reply_and_block(conn, msg, 60000,
							&err);
	if (!ret) {
		dbus_error_free(&err);
	} else {
		switch (dbus_message_get_type(ret)) {
		case DBUS_MESSAGE_TYPE_METHOD_RETURN:
			dbus_message_iter_init(ret, &iter);
			if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_STRING) {
				dbus_message_iter_get_basic(&iter, &flmsg);
				osso_log(LOG_INFO, " Folders List : %s \n", flmsg); 
				if(flmsg) {
					folders = g_strsplit(flmsg, ":", -1);
					for (idx = 0; (idx < STANFOLDERS_SIZE) && (folders[idx] != NULL); idx++) {
						standard_folders[idx].lname = g_strdup(folders[idx]);
					}
					g_strfreev(folders); 
				}
			}
			break;
		default:
			break;
		}
		dbus_message_unref(ret);
	}
	dbus_message_unref(msg);
	dbus_error_free(&err);

	standard_folders[0].fname = g_strdup(INBOX_DIR);
	standard_folders[1].fname = g_strdup(OUTBOX_DIR);
	standard_folders[2].fname = g_strdup(DRAFT_DIR);
	standard_folders[3].fname = g_strdup(SENT_DIR);
	standard_folders[4].fname = g_strdup(ARCHIVE);
	stan_folders_available = TRUE;
	return;
}

/**
	This method is called to get locale name from given folder name. 
	@param folder name name of the folder which locale name required
	@return the folder name for given locale name 
*/

static const gchar *ogs_email_fn_normal2locale(const gchar * fname)
{
	const gchar *out = NULL,*subfolder = NULL;
	static gchar out1[MAX_TRANSLATED_FOLDER_NAME_LEN];
	gint idx = 0;
	gint len = 0;
	
	/*Check if the input is a folder in rootdir or a subfolder */
	if((subfolder = g_utf8_strchr(fname,g_utf8_strlen(fname,-1),DIR_SEPRTR)) != NULL) {
		len = subfolder - fname;
	} else {
		len = g_utf8_strlen(fname, -1);
	}
	for (idx = 0; idx < STANFOLDERS_SIZE; idx++) {
		if(standard_folders[idx].fname == NULL) {
			continue;
		}
		if (strncmp(standard_folders[idx].fname, fname,len) == 0) {
			if(subfolder == NULL) {
				out = (const gchar *)standard_folders[idx].lname;
			} else {
				sprintf(out1,"%s%s",
					(const gchar *)standard_folders[idx].lname,
					subfolder);
				out = out1;
			}
		}
	}
	if(!out) {
		out = fname;
	}
	return out;
}

/**
	This method is called to get narmal name from locale folder name.
	@folder name name of the folder which locale name uis required
	@return the locale name for given folder 
*/
static const gchar *ogs_email_fn_locale2normal(const gchar * fname)
{
	const gchar *out = NULL;
	gint idx = 0;

	for (idx = 0; idx < STANFOLDERS_SIZE; idx++) {
		if(standard_folders[idx].lname == NULL) {
			continue;
		}
		if (strcmp(standard_folders[idx].lname, fname) == 0) {
			out = (const gchar *)standard_folders[idx].fname;
		}
	}
	if(!out) {
		out = fname;
	}
	return out;
}


/** 
	This method is called to search the mails based on the options
	selected by user.

	@param plugin Email Search plugin
	@param query search performed based on this query 
	@param context search context specific to eamil plugin
	@param fs file system
	@param category Email plugin category
	@options Email specific search options 
	@return nothing 
*/
static void
email_plugin_query(OgsPlugin * plugin,
		   const gchar * query,
		   OgsSearchContext * context,
		   OgsFileSystem * fs, OgsSearchCategory category, OgsSearchOptions * options)
{
	////////////////////////////////////////////////////////////////
	if (modest_is_default()) {
		email_plugin_query_modest(plugin, query, context,fs, category,
					  options);
		return;
	}
	////////////////////////////////////////////////////////////////

	EmailData data = { 0 };
	osso_log(LOG_INFO, "Email Plugin Query Starts here %s \n", query);
	if (!ogs_plugin_supports_category(plugin, category)) {
		osso_log(LOG_INFO, "Invalid Category\n");
		return;
	}

	ogs_email_get_email_standardfolders();

	if (query == NULL || strlen(query) == 0) {
		osso_log(LOG_INFO, "Selected Query is NULL\n");
		return;
	}

	data.context = context;
	data.plugin = plugin;
	data.category = category;
	data.options = options;
	data.query = (gchar *) query;
	data.sel_folder = (gchar *) ogs_search_options_get_folder(options);
	data.searcher = ogs_text_searcher_new(FALSE);
	ogs_text_searcher_parse_query(data.searcher, query);

	email_plugin_start_search(&data);
	ogs_text_searcher_free(data.searcher);
}

/** 
	This method is called to delete the selected search hit from list. 
	@param plugin Email Search plugin
	@param hit selected for deletion 
	@options error  if any error during delete operation 
	@return nothing 
*/

static gboolean email_plugin_delete_hit(OgsPlugin * plugin, OgsSearchHit * hit, GError ** error)
{
	////////////////////////////////////////////////////////////////
	if (modest_is_default()) 
		return email_plugin_delete_hit_modest(plugin, hit, error);
	////////////////////////////////////////////////////////////////

	gchar *msgid = NULL;
	osso_return_t rv = OSSO_OK;
	DBusMessage *message = NULL;
	DBusConnection *conn = NULL;
	dbus_uint32_t reason = 0;
	DBusError err;
	DBusMessage *ret = NULL;
	DBusMessageIter iter;
	
	osso_log(LOG_INFO, " email_plugin_open_delete_hit method called \n");
	dbus_error_init(&err);
	msgid = (gchar *) ogs_email_hit_get_msgid((OgsEmailHit *) hit);
	if (msgid == NULL) {
		g_set_error(error, 0, 0, "Could not delete mail");
		return FALSE;
	}
	if (email_plugin_ctx == NULL) {
		g_set_error(error, 0, 0, "Email Context is NULL");
		return FALSE;
	}
	conn = (DBusConnection *) osso_get_dbus_connection(email_plugin_ctx);
	if( !dbus_bus_start_service_by_name(conn, OSSO_EMAIL_SERVICE, 0, &reason, &err)) {
		dbus_error_free(&err);
		osso_log(LOG_INFO, "Error in activate Email\n");
		return FALSE;
	}

	message = dbus_message_new_method_call(OSSO_EMAIL_SERVICE,
					       OSSO_EMAIL_OBJ_PATH,
					       OSSO_EMAIL_SERVICE, DELETE_HIT_METHOD);
	if (message == NULL) {
		dbus_error_free(&err);
		return FALSE;
	}

	dbus_message_set_no_reply(message, FALSE);
	dbus_message_append_args(message, DBUS_TYPE_STRING, &msgid, DBUS_TYPE_INVALID);
	ret = dbus_connection_send_with_reply_and_block(conn, message, 60000, &err);
	if (ret == NULL) {
		osso_log(LOG_INFO, "reply and block return NULL \n");
		dbus_error_free(&err);
		dbus_message_unref(message);
		return FALSE;
	} else {
		osso_log(LOG_INFO, "%s reply has come for delete operation  \n", __FUNCTION__);
		if (dbus_message_get_type(ret) == DBUS_MESSAGE_TYPE_METHOD_RETURN) {
			dbus_message_iter_init(ret, &iter);
			if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_INT32) {
				dbus_message_iter_get_basic(&iter, &rv);
			}
		}
		dbus_message_unref(ret);
	}
	osso_log(LOG_INFO, "delete hit done \n");
	dbus_message_unref(message);
	return TRUE;
}

/** 
	This method is called to open the selected search hit from list. 
	@param plugin Email Search plugin
	@param hit selected for open 
	@options error  if any error during opening the hit 
	@return nothing 
*/

static gboolean email_plugin_open_hit(OgsPlugin * plugin, OgsSearchHit * hit, GError ** error)
{
	////////////////////////////////////////////////////////////////
	if (modest_is_default()) 
		return email_plugin_open_hit_modest(plugin, hit, error);
	////////////////////////////////////////////////////////////////

	gchar *msgid = NULL;
	osso_rpc_t retval = { 0 };
	osso_return_t rv = OSSO_OK;

	osso_log(LOG_INFO, " email_plugin_open_hit method called \n");
	msgid = (gchar *) ogs_email_hit_get_msgid((OgsEmailHit *) hit);

	if (msgid == NULL) {
		g_set_error(error, 0, 0, "Could not open mail");
		return FALSE;
	}
	if (email_plugin_ctx == NULL) {
		g_set_error(error, 0, 0, "Email Context is NULL");
		return FALSE;
	}
	rv = osso_rpc_run_with_defaults(email_plugin_ctx,
					OSSO_EMAIL,
					OPEN_HIT_METHOD,
					&retval, DBUS_TYPE_STRING, msgid, DBUS_TYPE_INVALID);
	osso_rpc_free_val (&retval);	
	if (rv != OSSO_OK) {
		return FALSE;
	}
	return TRUE;
}

/** 
	This method is called to perform search based on the user options 
	@param pl_data contains plugin data and user seleted options
	@param query search to be performed based on this query
	@options error  if any error during opening the hit
	@return nothing 
*/

static void email_plugin_start_search(EmailData * pl_data)
{
	gint ret_init_folder = -1;
	GSList *folder_list = NULL;
	GSList *cur = NULL;
	Folder *folder = NULL;
	const gchar *temp = NULL;
       	gchar *subfolder = NULL;
        gchar *rootfolder = NULL;
	gchar *userfolder = NULL;
	gint rl = 0;
	MmcHlrCallback *mmc_cb = NULL;
	
	if (pl_data == NULL) {
		osso_log(LOG_INFO, " Invalid search data \n");
		return;
	}
	if (!ogs_plugin_supports_category(pl_data->plugin, pl_data->category)) {
		return;
	}

	if (pl_data->query == NULL || strlen(pl_data->query) == 0) {
		return;
	}

	if (ogs_search_context_is_cancelled(pl_data->context)) {
		return;
	}
	ret_init_folder = init_folder();
	mmc_cb = mmc_init();
	account_init_list();
	folder_init_lists();
	gnome_vfs_init();

	folder_list = (GSList *) populate_folder_list();
	for (cur = folder_list; cur != NULL; cur = cur->next) {
		folder = (Folder *) cur->data;
		if ((folder == NULL) || (folder->name == NULL)) {
			continue;
		}
		if (ogs_search_context_is_cancelled(pl_data->context)) {
			break;
		}

		
		if ((pl_data->sel_folder == NULL) || (strlen(pl_data->sel_folder) == 0)) {
			email_plugin_search_sel_folder(pl_data, folder->name);
		} else {
			/* Get the main folder and compare with selected folder */
			subfolder = strchr(folder->name, DIR_SEPRTR);
			if (subfolder != NULL) {
				rl = strlen(folder->name) - strlen(subfolder);
				rootfolder = g_strndup(folder->name, rl);
			} else {
				rootfolder = g_strdup(folder->name);
			}
			/* Get the Localised str for  selected folder (Inbox Outbox...) */
			if (strncmp(pl_data->sel_folder, "USER:", strlen("USER:")) == 0) {
				userfolder = pl_data->sel_folder + strlen("USER:");
			} else if (strncmp(pl_data->sel_folder, "MAND:", strlen("MAND:")) == 0) {
				userfolder = pl_data->sel_folder + strlen("MAND:");
			} else {
				userfolder = pl_data->sel_folder;
			}

			temp = ogs_email_fn_locale2normal(userfolder);
			if ((temp != NULL) && (strcmp(temp, rootfolder) == 0)) {
				email_plugin_search_sel_folder(pl_data, folder->name);
			}
			g_free(rootfolder);
		}
	}
	mmc_uninit(mmc_cb);
	return;
}

/** 
	This method is called to perform search in the mail attachments
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information and attachments 
	information 
	@return TRUE for query found in attachment otherwise returns FALSE 
*/

static gboolean email_plugin_search_in_attachments(EmailData * pl_data, MsgInfo * msginfo)
{
	gboolean rv = FALSE;
	GSList *cur = NULL;
	AttachInfo *attach_info = NULL;

	if ((msginfo == NULL) || (msginfo->attach_list == NULL)) {
		return rv;
	}
	for (cur = msginfo->attach_list; cur != NULL; cur = cur->next) {
		attach_info = (AttachInfo *) cur->data;
		if ((attach_info == NULL) || (attach_info->filepath == NULL)) {
			continue;
		}
		rv = ogs_text_searcher_search_file(pl_data->searcher,
						   pl_data->context, attach_info->filepath, FALSE);

		unlink(attach_info->filepath);

		if (rv == TRUE) {
			osso_log(LOG_INFO, "query found in attachment \n");
		}
	}
	return rv;
}


/** 
	This method is called to get the body part in a file 
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information and attachments 
	information 
	@return body_file in case of success or returns NULL 
*/

static gchar *email_plugin_search_get_body_file(EmailData * pl_data, MsgInfo * msginfo)
{
	gint retval = FILE_PROCESS_SUCCESS;
	gchar *msgid = NULL;
	gchar *filepath = NULL;
	gchar *body_file = NULL;
	gchar *mime_file_name = NULL;
	gchar *enc_filename = NULL;
	gchar *dec_filename = NULL;
	gchar *mhtml_file = NULL;
	FILE *infp = NULL;
	MimeInfo *partinfo = NULL;
	MimeInfo *mimeinfo = NULL;
	gboolean part_body = TRUE;
	gboolean mhtml_body = FALSE;

	if ((msginfo == NULL) || (msginfo->header == NULL)) {
		return NULL;
	}

	msgid = msginfo->header->msgid;
	filepath = folder_get_message_file_path(msgid);

	if (is_file_entry_exist(filepath) == TRUE) {
		/* Get body content for mails which are not SMIME */
		if(msginfo->header->smime_type == SMIME_NONE) {
			mimeinfo = procmime_get_mimeinfo(msgid, filepath, FALSE, FALSE);
			body_file = g_strdup(mimeinfo->filename);
			procmime_mimeinfo_free_all(mimeinfo);
			g_free(filepath);
			return body_file;
		}
	}
	/* In case of IMAP check for only body downloaded mail */
	if(msginfo->header->msgtag.bodydownloaded != TRUE) {
		return NULL;
	}
	mime_file_name = g_strdup_printf("%s%c%s", filepath, SECTION_SEPARATOR, MIME_SUFFIX);
	if (is_file_entry_exist(mime_file_name) == TRUE) {
		mimeinfo = procmime_read_mimeinfo(mime_file_name);
	}
	g_free(mime_file_name);
	
	if(mimeinfo == NULL) {
		g_free(filepath);
		return NULL;
	}
	partinfo = procmime_get_text_ptr(mimeinfo, FALSE);
	if(partinfo == NULL) {
		body_file = g_strdup(mimeinfo->filename);
		procmime_mimeinfo_free_all(mimeinfo);
		g_free(filepath);
		return body_file;
	}

	enc_filename = g_strdup_printf("%s%c%s", filepath, SECTION_SEPARATOR, partinfo->section_number);
	if( (is_file_entry_exist(enc_filename) == FALSE)) {
		mhtml_file = g_strdup_printf("%s%c%s%c%s%c%s",get_mail_dir(), 
				G_DIR_SEPARATOR, INBOX_DIR, G_DIR_SEPARATOR,
				msgid, SECTION_SEPARATOR, MHTML_EXTENSION);

		if (is_file_entry_exist(mhtml_file) == TRUE) {
			mhtml_body = TRUE;
			part_body = FALSE;
		}
	}
	if(mhtml_body == TRUE) {
		mimeinfo->filename = decode_mhtml_file(mhtml_file,msgid, FALSE, FALSE);
	}else if( part_body == TRUE) {
		if( (infp = fopen(enc_filename, "r+")) != NULL) {
			GET_DECODE_FILENAME(msgid, partinfo, dec_filename);
			retval = procmime_get_content(partinfo, infp, dec_filename, TRUE, FALSE);
			fclose(infp);
			if (retval != FILE_PROCESS_FAIL) { 
				g_free(mimeinfo->filename);
				mimeinfo->filename = g_strdup(dec_filename);
			}
			g_free(dec_filename);
		} else { 
			procmime_check_and_set_filename(mimeinfo,NULL);
		}
	} 
	if(mimeinfo != NULL) {
		body_file = g_strdup(mimeinfo->filename);
	}
	g_free(enc_filename);
	g_free(mhtml_file);
	procmime_mimeinfo_free_all(mimeinfo);	
	return body_file;
}

/** 
	This method is called to perform search in the body part of mail 
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_search_in_body(EmailData * pl_data, MsgInfo * msginfo)
{
	gchar *outfile = NULL;
	gboolean rv = FALSE;

	if ((msginfo == NULL) || (msginfo->header == NULL)) {
		return rv;
	}
	outfile = email_plugin_search_get_body_file(pl_data, msginfo);
	if (outfile == NULL) {
		return rv;
	}

	rv = ogs_text_searcher_search_file(pl_data->searcher, pl_data->context, outfile, FALSE);
	ogs_text_searcher_reset(pl_data->searcher);
	g_free(outfile);

	if (rv == TRUE) {
		osso_log(LOG_INFO, "query(%s) found in body \n", pl_data->query);
	}
	return rv;
}

/** 
	This method is called to perform search in the subject field 
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_search_in_subject(EmailData * pl_data, MsgInfo * msginfo)
{
	gboolean rv = FALSE;

	if ((msginfo == NULL) || (msginfo->header == NULL)) {
		return rv;
	}
	if (msginfo->header->subject == NULL) {
		return rv;
	}
	rv = ogs_text_searcher_search_text(pl_data->searcher,
					   msginfo->header->subject,
					   strlen(msginfo->header->subject));
	ogs_text_searcher_reset(pl_data->searcher);
	if (rv == TRUE) {
		osso_log(LOG_INFO, "query(%s) found in subject\n", pl_data->query);
	}
	return rv;
}

/** 
	This method is called to perform search in the from, to, cc fields 
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_search_in_mailids(EmailData * pl_data, MsgInfo * msginfo)
{
	gboolean rv = FALSE;

	if ((msginfo == NULL) || (msginfo->header == NULL)) {
		return rv;
	}
	if (msginfo->header->from != NULL) {
		rv = ogs_text_searcher_search_text(pl_data->searcher,
						   msginfo->header->from,
						   strlen(msginfo->header->from));
		ogs_text_searcher_reset(pl_data->searcher);
		if (rv == TRUE) {
			osso_log(LOG_INFO, "query(%s) found in from\n", pl_data->query);
			return rv;
		}
	}
	if (msginfo->header->to != NULL) {
		rv = ogs_text_searcher_search_text(pl_data->searcher,
						   msginfo->header->to,
						   strlen(msginfo->header->to));
		ogs_text_searcher_reset(pl_data->searcher);
		if (rv == TRUE) {
			osso_log(LOG_INFO, "query(%s) found in to\n", pl_data->query);
			return rv;
		}
	}
	if (msginfo->header->cc != NULL) {
		rv = ogs_text_searcher_search_text(pl_data->searcher,
						   msginfo->header->cc,
						   strlen(msginfo->header->cc));
		ogs_text_searcher_reset(pl_data->searcher);
		if (rv == TRUE) {
			osso_log(LOG_INFO, "query(%s) found in cc\n", pl_data->query);
			return rv;
		}
	}
	return rv;
}

/** 
	This method is called to perform search in the complete mail 
	@param pl_data  contains plugin data and user seleted options 
	@param MsgInfo contains the mail related information 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_search_in_complete_mail(EmailData * pl_data, MsgInfo * msginfo)
{
	if ((msginfo == NULL) || (msginfo->header == NULL)) {
		return FALSE;
	}
	if (email_plugin_search_in_mailids(pl_data, msginfo) == TRUE) {
		return TRUE;
	}

	if (ogs_search_context_is_cancelled(pl_data->context)) {
		return FALSE;
	}

	if (email_plugin_search_in_subject(pl_data, msginfo) == TRUE) {
		return TRUE;
	}

	if (ogs_search_context_is_cancelled(pl_data->context)) {
		return FALSE;
	}

	if (email_plugin_search_in_body(pl_data, msginfo) == TRUE) {
		return TRUE;
	}
	if (email_plugin_search_in_attachments(pl_data, msginfo) == TRUE) {
		return TRUE;
	}
	return FALSE;

}

/** 
	This method is called to check the message file matching the date 
	and memory size criteria. 
	@param pl_data  contains plugin data and user seleted options
	@param msgidpath path of the selected mail for serching 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_verify_file_options(EmailData * pl_data, MsgHeader * msgheader)
{
	OgsSearchFields fields;
	GDate *mail_date = NULL;
	const GDate *date;
	gint size = 0;

	if ((pl_data == NULL) || (msgheader == NULL)) {
		return FALSE;
	}

	fields = ogs_search_options_get_valid_fields(pl_data->options);
	if (fields == OGS_SEARCH_OPTIONS_NONE) {
		return TRUE;
	}

	/* Check for File Size should be more than sel file size option */
	if (fields & OGS_SEARCH_OPTIONS_MIN_SIZE) {
		size = ogs_search_options_get_min_size(pl_data->options);
		if (msgheader->size < size) {
			return FALSE;
		}
	}

	mail_date = g_date_new();
	g_date_set_time(mail_date, msgheader->date_t);

	/* Check for Mail received date should be with in limits  */
	if (fields & OGS_SEARCH_OPTIONS_DATE_START) {
		date = ogs_search_options_get_date_start(pl_data->options);
		if (g_date_compare(mail_date, date) < 0) {
			g_date_free(mail_date);
			return FALSE;
		}
	}
	if (fields & OGS_SEARCH_OPTIONS_DATE_END) {
		date = ogs_search_options_get_date_end(pl_data->options);
		if (g_date_compare(mail_date, date) > 0) {
			g_date_free(mail_date);
			return FALSE;
		}
	}
	g_date_free(mail_date);
	return TRUE;
}


/** 
	This method is called to perform search user selected options.
	@param pl_data  contains plugin data and user seleted options
	@param msgidpath path of the selected mail for serching 
	@return TRUE for query found, otherwise returns FALSE 
*/

static gboolean email_plugin_verify_message_part(EmailData * pl_data, MsgInfo * msginfo)
{
	gboolean rv = FALSE;
	gint scope = 0;

	if ((pl_data == NULL) || (msginfo == NULL)) {
		osso_log(LOG_INFO, " Invalid search data \n");
		return FALSE;
	}
	if (pl_data->options == NULL) {
		return FALSE;
	}
	scope = ogs_search_options_get_scope(pl_data->options);
	switch (scope) {
	case OGS_SEARCH_SCOPE_ALL:	/*Entire body and attachment */
		rv = email_plugin_search_in_complete_mail(pl_data, msginfo);
		break;
	case OGS_SEARCH_SCOPE_ADDRESSES:	/* From, To, CC fileds */
		rv = email_plugin_search_in_mailids(pl_data, msginfo);
		break;
	case OGS_SEARCH_SCOPE_TITLES:	/* Titles, subjects only */
		rv = email_plugin_search_in_subject(pl_data, msginfo);
		break;
	case OGS_SEARCH_SCOPE_CONTENTS:	/* message bodies */
		rv = email_plugin_search_in_body(pl_data, msginfo);
		break;
	default:
		osso_log(LOG_INFO, "Invalid Search option = %d \n", scope);
	}
	return rv;
}

/** 
	This method is called to perform search in the specific folder 
	based on user selected options.
	@param pl_data  contains plugin data and user seleted options 
	@param folder name of the folder to be search performed. 
	@return nothing 
*/

static void email_plugin_search_sel_folder(EmailData * pl_data, gchar * folder)
{
	GSList *msgheader_list = NULL;
	GSList *msgheader_list_tmp = NULL;
	GSList *cur = NULL;
	GList *fl_list = NULL;
	gchar *acc_name = NULL;
	PrefsAccount *account = NULL;
	MsgInfo *msginfo = NULL;
	MsgHeader *msgheader = NULL;
	gchar *filename = NULL;
	gboolean attachment = FALSE;
	gboolean archive_flag = FALSE;
	gboolean unread = FALSE;
	gint got_mails = 0;
	gint total_mails = 0;

	osso_log(LOG_INFO, "email_plugin_search_sel_folder %s starts here \n", folder);

	if ((pl_data == NULL) || (folder == NULL)) {
		osso_log(LOG_INFO, " Invalid search data \n");
		return;
	}
	if (pl_data->query == NULL || strlen(pl_data->query) == 0) {
		return;
	}
	if (ogs_search_context_is_cancelled(pl_data->context)) {
		return;
	}
	/* Get the message header list */
	if ((fl_list = folder_get_file_list(folder)) == NULL) {
		return;
	}
	
	total_mails = g_list_length(fl_list);
	do
	{
		msgheader_list_tmp = folder_populate_from_list(fl_list, got_mails, folder);
		msgheader_list = g_slist_concat(msgheader_list,msgheader_list_tmp);
		got_mails += g_slist_length(msgheader_list_tmp);
		
	}while (got_mails < total_mails);
	g_list_foreach(fl_list,(GFunc) gnome_vfs_file_info_unref,NULL);
	g_list_free(fl_list);
	if (msgheader_list == NULL) {
		return;
	}
	for (cur = msgheader_list; cur != NULL; cur = cur->next) {
		if (cur->data == NULL) {
			continue;
		}
		if (ogs_search_context_is_cancelled(pl_data->context)) {
			break;
		}
		msgheader = (MsgHeader *) cur->data;
		if (msgheader->msgid == NULL) {
			continue;
		}
		acc_name = (gchar *) get_account_name(msgheader->msgid);
		if( acc_name ) {
			account = (PrefsAccount *) account_get_account_cb(acc_name);
			g_free(acc_name);
		}
		if(account == NULL) {
			gchar *msg_file_name = NULL;
			msgheader->msgtag.protocol = PROTO_INVALID;
			msgheader->msgtag.fullydownloaded = TRUE;
			msgheader->msgtag.bodydownloaded = TRUE;
			msg_file_name = folder_get_message_file_path(msgheader->msgid);
			if (procmime_check_attachments(msg_file_name, FALSE) == TRUE) {
				 msgheader->flags.perm_flags |= MSG_ATTACH;
			}
			msgheader->msgtag.recvtype = BODY_ATTCH;
		 	g_free(msg_file_name);
		} else {
			msgheader->msgtag.protocol = account->receive_settings.protocol;
			msgheader->msgtag.recvtype = account->receive_settings.recv_type;
			procmime_folder_set_header(msgheader, account);
		}
      		/** check for archive folder **/
		if (check_archive(msgheader->msgid, &archive_flag) == FILE_PROCESS_FAIL) {
			continue;
		}
		filename = folder_get_message_file_path(msgheader->msgid);
		if (is_file_exist(filename) != TRUE) {
			gchar *newfilename = NULL;
			if(  msgheader->msgtag.protocol == PROTO_POP3) { 
				/* POP3 header only mail */
				newfilename = g_strdup_printf("%s%c%s", filename, DOT, MSG_HDR);
			}else if (msgheader->msgtag.protocol == PROTO_IMAP4) { 
				/* for IMAP header/partially download */
				newfilename = g_strdup_printf("%s%c%s", filename, DOT, HEADER_SECTION);
			}
			g_free(filename);
			filename = newfilename;
		}
		/* Check file options match the selected options */
		if (email_plugin_verify_file_options(pl_data, msgheader) == FALSE) {
			g_free(filename);
			del_arch_file(msgheader->msgid, archive_flag);
			continue;
		}
		
		msginfo = folder_parse_msg(filename);
		if ((msginfo == NULL) || (msginfo->header == NULL)) {
			del_arch_file(msgheader->msgid, archive_flag);
			osso_log(LOG_INFO, "MsgInfo NULL = %s \n", filename); 
			g_free(filename);
			procmsg_msginfo_free(msginfo);
			continue;
		}
		msginfo->header->msgtag = msgheader->msgtag;
		msginfo->header->flags = msgheader->flags;
		g_free(msginfo->header->msgid);
		msginfo->header->msgid = g_strdup(msgheader->msgid);
		g_free(filename);

		if (email_plugin_verify_message_part(pl_data, msginfo) == TRUE) {
			attachment = FALSE;
			unread = FALSE;
			if ((msginfo->header->flags.perm_flags & MSG_UNREAD) != 0) {
				unread = TRUE;
			}
			if ((msgheader->flags.perm_flags & MSG_ATTACH) != 0) {
				attachment = TRUE;
			}
			if(!msginfo->header->subject || strlen(msginfo->header->subject) == 0) {
				msginfo->header->subject = g_strdup(DGETTEXT_GS("srch_ia_no_subject"));
		        }
			
			email_plugin_add_hit(pl_data,
					     msginfo->header->from,
					     msginfo->header->subject,
					     folder,
					     msginfo->header->msgid,
					     msginfo->header->date_t,
					     attachment, unread, (guint) msginfo->header->size);
		}
		del_arch_file(msginfo->header->msgid, archive_flag);
		procmsg_msginfo_free(msginfo);
	}
	procmsg_msgheader_list_free(msgheader_list);
}

/** 
	This method is called to to send new hit siganl to main application.
	@param contact nemw of the sender address
	@param  subject suject field in the new found hit
	@param  folder  folder name of the new hit exists 
	@param  msgsid  messageid of the new found hit 
	@param  timestamp new fond hit time stamp 
	@param  has_attachment is set TRUE in case of new hit found contains 
	attchment 
	@param  is_unread is the set to TRUE message is not read 
*/
static void
email_plugin_add_hit(EmailData * pl_data,
		     const gchar * contact,
		     const gchar * subject,
		     const gchar * folder,
		     const gchar * msgid,
		     glong timestamp, gboolean has_attachment, gboolean is_unread, guint msize)
{
	OgsSearchHit *hit = NULL;

	osso_log(LOG_INFO, "Add found entry : %s \n", msgid);

	if (pl_data == NULL) {
		return;
	}
        hit = g_object_new(OGS_TYPE_EMAIL_HIT, 
			"name",        	subject, 
			"mime-type",   	"email", 
			"folder", 	ogs_email_fn_normal2locale(folder), 
			"msgid", 	msgid,
			"timestamp", 	timestamp,
		       	"size",     	(gulong)msize, 
			"category",      pl_data->category, 
			"contact",       contact, 
			"has_attachment", has_attachment, 
			"is_unread",     is_unread, 
			NULL);
			
	ogs_plugin_new_hit(pl_data->plugin, hit, pl_data->context);
	g_object_unref(hit);
}

/**
	This fuction registerd as a call back function to Mmc Hlr.
	MmcHlr invokes this function when mmc is mounted on the device
	@param volume_monitor pointer to the GnomeVFSVolumeMonitor information
	@param volume  pointer to the GnomeVFSVolume information
	@return on successful operation returns OSSO_PACK_OK. Otherwise 
	returns errorcode 
*/

static gint
mmc_hlr_volume_mounted_cb(GnomeVFSVolumeMonitor * volume_monitor, GnomeVFSVolume * volume)
{
	gchar *path = NULL;
	gchar *mmc_path = NULL;
	GnomeVFSURI *mmc_uri;

	path = gnome_vfs_volume_get_activation_uri(volume);
	mmc_uri = gnome_vfs_uri_new(path);
	g_free(path);
	mmc_path = (gchar *) gnome_vfs_uri_get_path(mmc_uri);
	if (strcmp(mmc_path, MMC_PATH) != 0) {
		g_free(mmc_path);
		return -1;
	}
	update_mmc_flag(TRUE, mmc_path);
	g_free(mmc_path);

	return 0;
}


/**
	This fuction registerd as a call back function to Mmc Hlr.
	MmcHlr invokes this function when mmc is unmounted on the device 
	@param volume_monitor pointer to the GnomeVFSVolumeMonitor information 
	@param volume  pointer to the GnomeVFSVolume information 
	@return on success returns 0 
*/

static gint
mmc_hlr_volume_unmounted_cb(GnomeVFSVolumeMonitor * volume_monitor, GnomeVFSVolume * volume)
{
	gchar *path = NULL;
	gchar *mmc_path = NULL;
	GnomeVFSURI *mmc_uri;

	path = gnome_vfs_volume_get_activation_uri(volume);
	mmc_uri = gnome_vfs_uri_new(path);
	g_free(path);
	mmc_path = (gchar *) gnome_vfs_uri_get_path(mmc_uri);
	if (strcmp(mmc_path, MMC_PATH) != 0) {
		g_free(mmc_path);
		return -1;
	}
	update_mmc_flag(FALSE, mmc_path);
	g_free(mmc_path);
	return 0;
}

/**
	This function will be called to register the mmc call back
	@param cb_ptr    pointer to the MmcHlrCallback
	@return flag to indicate whether regsitering was successful
*/

static MmcHlrCallback *mmc_init(void)
{
	MmcHlrCallback *mmc_hlrCb = NULL;
	mmc_hlrCb = g_new0(MmcHlrCallback, 1);
	mmc_hlrCb->mmc_hlr_volume_mounted_cb = mmc_hlr_volume_mounted_cb;
	mmc_hlrCb->mmc_hlr_volume_unmounted_cb = mmc_hlr_volume_unmounted_cb;

	mmc_hlr_init(mmc_hlrCb);

	return mmc_hlrCb;
}

/**
	This function will be called to free memory allocated during mmc_init 
	@param none 
	@return none 
 */

static void mmc_uninit(MmcHlrCallback * mmc_hlrCb)
{
	mmc_hlrCb->mmc_hlr_volume_mounted_cb = NULL;
	mmc_hlrCb->mmc_hlr_volume_unmounted_cb = NULL;
	g_free(mmc_hlrCb);
	mmc_hlrCb = NULL;
}
